@@ -3,6 +3,7 @@ from __future__ import annotations
 import os
 from contextlib import ExitStack
 from dataclasses import dataclass, field
+from datetime import datetime, timedelta
 from enum import Enum
 from typing import Any, Callable, Generic, Iterator, TypeVar
 
@@ -160,6 +161,22 @@ class HostedRunResult(Generic[ResultT]):
     result: ResultT | None = None
 
 
+@dataclass
+class UserKeyInfo:
+    key_id: str
+    created_at: datetime
+
+
+@dataclass
+class WorkerStatus:
+    worker_id: str
+    start_time: datetime
+    end_time: datetime
+    duration: timedelta
+    user_id: str
+    machine_type: str
+
+
 @from_grpc.register(isolate_proto.HostedRunStatus)
 def _from_grpc_hosted_run_status(
     message: isolate_proto.HostedRunStatus,
@@ -184,9 +201,6 @@ def _from_grpc_hosted_run_result(
     )
 
 
-_SUPPORTED_MACHINE_TYPES = ["XS", "S", "M", "L", "GPU"]
-
-
 def _get_run_id(run: ScheduledRun | str) -> str:
     if isinstance(run, ScheduledRun):
         return run.run_id
@@ -199,9 +213,6 @@ class MachineRequirements:
     machine_type: str
     keep_alive: int = KOLDSTART_DEFAULT_KEEP_ALIVE
 
-    def __post_init__(self):
-        assert self.machine_type in _SUPPORTED_MACHINE_TYPES
-
 
 @dataclass
 class KoldstartConnection:
@@ -239,10 +250,16 @@ class KoldstartConnection:
         response = self.stub.CreateUserKey(request)
         return response.key_secret, response.key_id
 
-    def list_user_keys(self) -> list[isolate_proto.UserKeyInfo]:
+    def list_user_keys(self) -> list[UserKeyInfo]:
         request = isolate_proto.ListUserKeysRequest()
         response: isolate_proto.ListUserKeysResponse = self.stub.ListUserKeys(request)
-        return list(response.user_keys)
+        return [
+            UserKeyInfo(
+                key.key_id,
+                isolate_proto.datetime_from_timestamp(key.created_at),
+            )
+            for key in response.user_keys
+        ]
 
     def revoke_user_key(self, key_id) -> None:
         request = isolate_proto.RevokeUserKeyRequest(key_id=key_id)
@@ -257,7 +274,6 @@ class KoldstartConnection:
         assert isinstance(
             configuration_options, dict
         ), "configuration_options must be a dict"
-
         struct = isolate_proto.Struct()
         struct.update(configuration_options)
 
@@ -372,3 +388,18 @@ class KoldstartConnection:
         )
         response = self.stub.GetScheduledActivationLogs(request)
         return response.raw_logs
+
+    def list_worker_status(self, user_id: str | None = None) -> list[WorkerStatus]:
+        request = isolate_proto.WorkerStatusListRequest(user_id=user_id)
+        response = self.stub.WorkerStatusList(request)
+        return [
+            WorkerStatus(
+                ws.worker_id,
+                isolate_proto.datetime_from_timestamp(ws.start_time),
+                isolate_proto.datetime_from_timestamp(ws.end_time),
+                isolate_proto.timedelta_from_duration(ws.duration),
+                ws.user_id,
+                ws.machine_type,
+            )
+            for ws in response.worker_status
+        ]
